'use strict';

var debug = require('debug')('geteventstore:checkStreamExists');
var req = require('request-promise');
var Promise = require('bluebird');
var assert = require('assert');
var url = require('url');

var baseErr = 'Check Stream Exists - ';

module.exports = function (config) {
    var buildUrl = function buildUrl(streamName) {
        var urlObj = JSON.parse(JSON.stringify(config));
        urlObj.pathname = '/streams/' + streamName + '/head/backward/1';
        return url.format(urlObj);
    };

    return function (streamName) {
        return Promise.resolve().then(function () {
            assert(streamName, baseErr + 'Stream Name not provided');

            var options = {
                uri: buildUrl(streamName),
                method: 'GET',
                json: true,
                headers: {
                    "Content-Type": "application/vnd.eventstore.events+json"
                },
                timeout: config.timeout
            };

            debug('', 'Options: %j', options);
            return req(options).then(function (response) {
                debug('', 'Response: %j', response);
                return true;
            }).catch(function (err) {
                if (err.statusCode !== 404) return Promise.reject(err);
                return false;
            });
        });
    };
};