'use strict';

var debug = require('debug')('geteventstore:assertProjection');
var req = require('request-promise');
var Promise = require('bluebird');
var assert = require('assert');
var _ = require('lodash');
var url = require('url');

var baseErr = 'Assert Projection - ';

var doesProjectionExist = function doesProjectionExist(config, name) {
    return Promise.resolve().then(function () {
        var getAllProjectionsInfo = require('./getAllProjectionsInfo')(config);
        return getAllProjectionsInfo().then(function (projectionsInfo) {
            var projection = _.find(projectionsInfo.projections, function (projection) {
                return projection.name === name;
            });
            if (projection) return true;
            return false;
        });
    });
};

var buildCreateOptions = function buildCreateOptions(config, name, projectionContent, mode, enabled, emitEnabled, checkpointsEnabled) {
    var urlObj = JSON.parse(JSON.stringify(config));
    urlObj.pathname = '/projections/' + mode;
    var uri = url.format(urlObj);

    var options = {
        uri: uri,
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        qs: {
            name: name,
            enabled: enabled ? 'yes' : 'no',
            emit: emitEnabled ? 'yes' : 'no',
            checkpoints: checkpointsEnabled ? 'yes' : 'no'
        },
        body: projectionContent
    };

    return options;
};

var buildUpdateOptions = function buildUpdateOptions(config, name, projectionContent, emitEnabled) {
    var urlObj = JSON.parse(JSON.stringify(config));
    urlObj.pathname = '/projection/' + name + '/query';
    var uri = url.format(urlObj);

    var options = {
        uri: uri,
        method: 'PUT',
        headers: {
            'Content-Type': 'application/json'
        },
        qs: {
            emit: emitEnabled ? 'yes' : 'no'
        },
        body: projectionContent
    };

    return options;
};

module.exports = function (config) {
    return function (name, projectionContent, mode, enabled, checkpointsEnabled, emitEnabled) {
        return Promise.resolve().then(function () {
            assert(name, baseErr + 'Name not provided');
            assert(projectionContent, baseErr + 'Projecion Contnet not provided');

            mode = mode || 'continuous';
            enabled = enabled || true;
            checkpointsEnabled = mode === 'continuous' ? true : checkpointsEnabled || false;
            emitEnabled = emitEnabled || false;

            return doesProjectionExist(config, name).then(function (projectionExists) {
                debug('', 'Projection Exists: %j', projectionExists);
                var options = {};

                if (!projectionExists) options = buildCreateOptions(config, name, projectionContent, mode, enabled, emitEnabled, checkpointsEnabled);else options = buildUpdateOptions(config, name, projectionContent, emitEnabled);

                debug('', 'Options: %j', options);
                return req(options).then(function (response) {
                    debug('', 'Response: %j', response);
                    return JSON.parse(response);
                });
            });
        });
    };
};