'use strict';

var debug = require('debug')('geteventstore:writeEvents');
var req = require('request-promise');
var Promise = require('bluebird');
var assert = require('assert');
var url = require('url');

var baseErr = 'Write Events - ';

module.exports = function (config) {
    var buildUrl = function buildUrl(streamName) {
        var urlObj = JSON.parse(JSON.stringify(config));
        urlObj.pathname = '/streams/' + streamName;
        return url.format(urlObj);
    };

    return function (streamName, events, options) {
        return Promise.resolve().then(function () {
            assert(streamName, baseErr + 'Stream Name not provided');
            assert(events, baseErr + 'Events not provided');
            assert.equal(true, events.constructor === Array, baseErr + 'Events should be an array');

            if (events.length === 0) return;

            options = options || {};
            options.expectedVersion = options.expectedVersion || -2;

            var reqOptions = {
                uri: buildUrl(streamName),
                headers: {
                    "Content-Type": "application/vnd.eventstore.events+json",
                    "ES-ExpectedVersion": options.expectedVersion
                },
                method: 'POST',
                body: events,
                json: true,
                timeout: config.timeout
            };
            debug('', 'Write events: %j', reqOptions);
            return req(reqOptions);
        });
    };
};