'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = require('debug')('geteventstore:eventEnumerator');
var connectionManager = require('./connectionManager');
var mapEvents = require('./utilities/mapEvents');
var Promise = require('bluebird');
var assert = require('assert');

var baseErr = 'Event Enumerator - ';

var getNextBatch = function getNextBatch(config, streamName, state, length, direction, resolveLinkTos) {
    state.isFirstEnumeration = false;
    return Promise.resolve().then(function () {
        assert(streamName, baseErr + 'Stream Name not provided');

        return connectionManager.create(config).then(function (connection) {
            function handleResult(result) {
                debug('', 'Result: %j', result);

                state.nextEventNumber = result.nextEventNumber;
                return {
                    isEndOfStream: result.isEndOfStream,
                    events: mapEvents(result.events)
                };
            }

            if (direction === 'forward') return connection.readStreamEventsForward(streamName, state.nextEventNumber, length, resolveLinkTos, config.credentials).then(handleResult);
            return connection.readStreamEventsBackward(streamName, state.nextEventNumber, length, resolveLinkTos, config.credentials).then(handleResult);
        });
    });
};

var esDirectionWorkaroundHandler = function esDirectionWorkaroundHandler(direction) {
    var wasSwopped = false;

    if (direction === 'forward') {
        wasSwopped = true;
        direction = 'backward';
    }

    return {
        direction: direction,
        swopResult: function swopResult(state, length, result) {
            if (wasSwopped) {
                state.nextEventNumber += length + 1;
                result.events.reverse();
            }
            return result;
        }
    };
};

var stateHandler = function stateHandler(direction) {
    var Handler = function () {
        function Handler() {
            _classCallCheck(this, Handler);

            this.isFirstEnumeration = true;
            this.setToFirst();
        }

        _createClass(Handler, [{
            key: 'setToFirst',
            value: function setToFirst() {
                this.nextEventNumber = direction === 'forward' ? 0 : -1;
            }
        }, {
            key: 'setToLast',
            value: function setToLast(length) {
                this.nextEventNumber = direction === 'forward' ? -1 : length - 1;
            }
        }, {
            key: 'setToPrevious',
            value: function setToPrevious(length) {
                if (!this.isFirstEnumeration) this.adjustByLength(length);
            }
        }, {
            key: 'keepInBoundsAdjustment',
            value: function keepInBoundsAdjustment(length) {
                if (direction === 'backward') return length;

                var adjustment = length;
                if (this.nextEventNumber < -1) {
                    adjustment -= Math.abs(this.nextEventNumber);
                    this.nextEventNumber = 0;
                }

                return adjustment;
            }
        }, {
            key: 'adjustByLength',
            value: function adjustByLength(length) {
                this.nextEventNumber += direction === 'forward' ? length * -1 : length;
            }
        }]);

        return Handler;
    }();

    return new Handler();
};

module.exports = function (config) {
    return function (streamName, direction, resolveLinkTos) {
        direction = direction || 'forward';
        resolveLinkTos = resolveLinkTos === undefined ? true : resolveLinkTos;
        var state = stateHandler(direction);

        return {
            first: function first(length) {
                state.setToFirst();
                return getNextBatch(config, streamName, state, length, direction, resolveLinkTos);
            },
            last: function last(length) {
                state.setToLast(length);

                var handler = esDirectionWorkaroundHandler(direction);
                return getNextBatch(config, streamName, state, length, handler.direction, resolveLinkTos).then(function (result) {
                    return handler.swopResult(state, length, result);
                });
            },
            previous: function previous(length) {
                state.setToPrevious(length);
                length = state.keepInBoundsAdjustment(length);

                return getNextBatch(config, streamName, state, length, direction, resolveLinkTos).then(function (result) {
                    state.adjustByLength(length);
                    return result;
                });
            },
            next: function next(length) {
                return getNextBatch(config, streamName, state, length, direction, resolveLinkTos);
            }
        };
    };
};